package com.game.business.task;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.game.business.domain.*;
import com.game.business.service.*;
import com.game.common.constant.finance.FinTranType1;
import com.game.common.constant.finance.FinTranType3;
import com.game.common.entity.KeyValue;
import com.game.common.utils.DateUtils;
import com.game.common.utils.StringUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Component("appUserCountTask")
@Log4j2
public class AppUserCountTask {

    @Autowired
    private IAppUsersChargeService appUsersChargeService;

    @Autowired
    private IAppUserService appUserService;

    @Autowired
    private IAppUsersCashrecordService appUsersCashrecordService;

    @Autowired
    private IFinTranRecordService finTranRecordService;

    @Autowired
    private IAppUserLiveDividedRecordService appUserLiveDividedRecordService;

    @Autowired
    private IAppUserGameRecordCountService appUserGameRecordCountService;

    @Autowired
    private IAppGameBettingService appGameBettingService;

    @Autowired
    private IAppUserCountService appUserCountService;

    @Autowired
    private IAppUserGameCountService appUserGameCountService;

    @Autowired
    private IAppUserAgentService appUserAgentService;

    @Autowired
    private IAppUserCountDividendService appUserCountDividendService;


    /**
     * 统计用户前一天
     * */
    public void statistics(String dateTime){
        log.info("开始统计用户每日直播、游戏、佣金");
        try {
            Date curDate = DateUtil.offsetDay(DateUtils.getNowDate(), -1);
            if (StringUtils.isNotBlank(dateTime)) {
                curDate = DateUtil.parse(dateTime, "yyyy-MM-dd");
            } else {
                dateTime = DateUtils.parseDateToStr("yyyy-MM-dd", curDate);
            }
            AppUser queryUser = new AppUser();
            queryUser.setAgentFlag(1);
            List<AppUser> list = appUserService.selectAppUserList(queryUser);//查询所有代理用户
            List<AppUsersCharge> appUsersChargeList = appUsersChargeService.getChargeList(null, dateTime.concat(" 00:00:00"), dateTime.concat(" 23:59:59"));
            List<AppUsersCashrecord> appUsersCashrecords = appUsersCashrecordService.getUserCashList(null, dateTime.concat(" 00:00:00"), dateTime.concat(" 23:59:59"));
            List<FinTranRecord> finTranRecordList = finTranRecordService.getUserRecordList(null, dateTime.concat(" 00:00:00"), dateTime.concat(" 23:59:59"));
            List<AppUserLiveDividedRecord> liveDividedRecordList = appUserLiveDividedRecordService.selectByDate(null, dateTime.concat(" 00:00:00"), dateTime.concat(" 23:59:59"));
            List<AppUserGameRecordCount> gameRecordCountList = appUserGameRecordCountService.selectByDate(null, dateTime.concat(" 00:00:00"), dateTime.concat(" 23:59:59"));
            List<AppGameBetting> gameBettingList = appGameBettingService.selectListByDate(null, dateTime.concat(" 00:00:00"), dateTime.concat(" 23:59:59"),null);
            if (null != list && list.size() > 0) {
                List<AppUserCount> appUserCountList = new ArrayList<>();
                String finalDateTime = dateTime;
                list.forEach(appUser -> {
                    AppUserCount appUserCount = new AppUserCount();
                    appUserCount.setUserId(appUser.getUserid());
                    appUserCount.setStrDate(finalDateTime);
                    appUserCount.setAgentUserId(appUser.getPid());
                    //充值总额
                    appUserCount.setRechargeAmount(appUsersChargeList.stream().filter(e -> e.getUid().equals(appUser.getUserid())).mapToDouble(e -> e.getMoney().doubleValue()).sum());
                    //提现金额
                    appUserCount.setWithdrawalAmount(appUsersCashrecords.stream().filter(e -> e.getUid().equals(appUser.getUserid())).mapToDouble(e -> e.getMoney().doubleValue()).sum());
                    //充值服务费/手续费
                    appUserCount.setRechargeCommission(appUsersChargeList.stream().filter(e -> e.getUid().equals(appUser.getUserid())).mapToDouble(e -> e.getPlatformService()).sum());
                    //充值赠送余额
                    appUserCount.setRechargeGive(finTranRecordList.stream().filter(e -> {
                        return e.getUid().equals(appUser.getUserid()) &&
                                e.getTranType1().intValue() == FinTranType1.U_Income_Coin_Balance.getType() &&
                                e.getTranType3().intValue() == FinTranType3.CHARGE_IN_REWARD.getType();
                    }).mapToDouble(e->e.getCoinChange()).sum());
                    //提现服务费/手续费
                    appUserCount.setWithdrawalCommission(appUsersCashrecords.stream().filter(e -> e.getUid().equals(appUser.getUserid())).mapToDouble(e -> e.getService().doubleValue()).sum());
                    //直播消费金额
                    Map<String, KeyValue> type1Keys = Arrays.stream(FinTranType1.getKeyValues(1)).collect(Collectors.toMap(e -> e.a, Function.identity(), (k1, k2) -> k2)); //支出交易类型
                    Map<String, KeyValue> type3Keys = Arrays.stream(FinTranType3.getKeyValues(1)).collect(Collectors.toMap(e -> e.a, Function.identity(), (k1, k2) -> k2)); //支出交易类型
                    appUserCount.setLiveUseAmount(finTranRecordList.stream().filter(e -> {
                        return e.getUid().equals(appUser.getUserid()) &&
                                null != type1Keys.get(String.valueOf(e.getTranType1().intValue())) &&
                                null != type3Keys.get(String.valueOf(e.getTranType3().intValue()));
                    }).mapToDouble(e -> dealMoney(e.getCoinChange())).sum());
                    //直播佣金
                    appUserCount.setLiveCommission(liveDividedRecordList.stream().filter(e -> e.getUserId().equals(appUser.getUserid())).mapToDouble(e -> e.getMoney().doubleValue()).sum());
                    //游戏佣金
                    appUserCount.setGameCommission(gameRecordCountList.stream().filter(e -> e.getUserId().equals(appUser.getUserid())).mapToDouble(e -> e.getCommission()).sum());
                    //游戏输
                    appUserCount.setGameLoseAmount(gameBettingList.stream().filter(
                            e -> e.getUserId().equals(appUser.getUserid()) && e.getIsWinning().intValue() == 2
                    ).mapToDouble(e ->
                            e.getBettingAmount()
                    ).sum());
                    //游戏赢
                    appUserCount.setGameWinAmount(gameBettingList.stream().filter(
                            e -> e.getUserId().equals(appUser.getUserid()) && e.getIsWinning().intValue() == 1
                    ).mapToDouble(e ->
                            e.getBettingAmount() * e.getBettingMultiple()
                    ).sum());
                    appUserCountService.createAppUserCount(appUserCount);
                });
            }

            //统计当日用户游戏输赢
            if (null != gameBettingList && gameBettingList.size() > 0) {
                Map<String, List<AppGameBetting>> map = gameBettingList.stream().
                        collect(Collectors.groupingBy(
                                e -> String.valueOf(e.getUserId()).concat("_").
                                        concat(String.valueOf(e.getGameId())).
                                        concat("_").concat(String.valueOf(e.getIsWinning()))
                        ));
                for (String key : map.keySet()) {
                    List<AppGameBetting> appGameBettingList = map.get(key);
                    Long userId = Long.valueOf(key.split("_")[0]);
                    Long gameId = Long.valueOf(key.split("_")[1]);
                    AppUserGameCount appUserGameCount = new AppUserGameCount();
                    appUserGameCount.setUserId(userId);
                    appUserGameCount.setGameId(gameId);
                    appUserGameCount.setStrDate(dateTime);
                    appUserGameCount.setGameLoseAmount(appGameBettingList.stream().filter(e -> 2 == e.getIsWinning()).mapToDouble(
                            e -> e.getBettingAmount()
                    ).sum());
                    appUserGameCount.setGameWinAmount(appGameBettingList.stream().filter(e -> 2 == e.getIsWinning()).mapToDouble(
                            e -> e.getBettingAmount() * e.getBettingMultiple()
                    ).sum());
                    appUserGameCount.setGameCommission(gameRecordCountList.stream().filter(
                            e -> e.getUserId().equals(userId) && e.getGameId().equals(gameId)
                    ).mapToDouble(e -> e.getCommission()).sum());//自己得到的佣金
                    appUserGameCountService.save(appUserGameCount);
                }
            }
        }catch (Exception e){
            log.info("用户每日直播、游戏、佣金统计异常：{}",e.getMessage());
        }
        log.info("用户每日直播、游戏、佣金统计完成");

    }

    /**
     * 计算分红
     * @param dateTime 格式yyyy-MM-dd
     * */
    public void calculateDividends(String dateTime){
        try {
            log.info("开始每周代理分红");
            Date curDate = DateUtil.offsetDay(DateUtils.getNowDate(), -1);
            if (StringUtils.isNotBlank(dateTime)) {
                curDate = DateUtil.parse(dateTime, "yyyy-MM-dd");
            } else {
                dateTime = DateUtils.parseDateToStr("yyyy-MM-dd", curDate);
            }
            Date beginTime = DateUtil.beginOfWeek(curDate);
            Date endTime = DateUtil.endOfWeek(curDate);
            //查询所有代理用户
            AppUser queryUser = new AppUser();
            queryUser.setAgentFlag(1);
            List<AppUser> list = appUserService.selectAppUserList(queryUser);
            if (null != list && list.size() > 0) {
                list.forEach(appUser -> {
                    //查询用户所有下级本周盈亏
                    LambdaQueryWrapper<AppUserCount> queryWrapper = new LambdaQueryWrapper<>();
                    queryWrapper.ge(AppUserCount::getStrDate, beginTime);
                    queryWrapper.le(AppUserCount::getStrDate, endTime);
                    queryWrapper.eq(AppUserCount::getAgentUserId, appUser.getUserid());
                    List<AppUserCount> appUserCountList = appUserCountService.list(queryWrapper);
                    if (null != appUserCountList && appUserCountList.size() > 0) {
                        //游戏输钱
                        double loseAmount = appUserCountList.stream().mapToDouble(e -> e.getGameLoseAmount()).sum();
                        //游戏赢钱
                        double winAmount = appUserCountList.stream().mapToDouble(e -> e.getGameWinAmount()).sum();
                        //提现手续费
                        double withdrawalCommission = appUserCountList.stream().mapToDouble(e -> e.getWithdrawalCommission()).sum();
                        //游戏佣金
                        double gameCommission = appUserCountList.stream().mapToDouble(e -> e.getGameCommission()).sum();
                        //直播佣金
                        double liveCommission = appUserCountList.stream().mapToDouble(e -> e.getLiveCommission()).sum();
                        //充值赠送
                        double rechargeGive = appUserCountList.stream().mapToDouble(e -> e.getRechargeGive()).sum();

                        //计算分红金额 （输-赢）
//                        double money = (loseAmount - winAmount) - withdrawalCommission - gameCommission - liveCommission;
                        double money = (loseAmount - winAmount)  - gameCommission - rechargeGive;
                        if (money > 0) {
                            //金额大于0 满足分红

                            //查询用户所有下级
                            LambdaQueryWrapper<AppUserAgent> userAgentLambdaQueryWrapper = new LambdaQueryWrapper<>();
                            userAgentLambdaQueryWrapper.eq(AppUserAgent::getPid, appUser.getUserid());
                            userAgentLambdaQueryWrapper.eq(AppUserAgent::getAuditStatus, 1);
                            List<AppUserAgent> appUserAgentList = appUserAgentService.list(userAgentLambdaQueryWrapper);
                            if (null != appUserAgentList && appUserAgentList.size() > 0) {
                                appUserAgentList.forEach(userAgent -> {
                                    if (userAgent.getDividendGuaranteeRate() <= 0.00) {
                                        log.info("用户{}，团队亏损{}，或未设置分红比例，停止分红", userAgent.getUserId(), money);
                                        return;
                                    }
                                    double rate = userAgent.getDividendGuaranteeRate();
                                    double dividends = money * rate/100;
                                    AppUserCountDividend appUserCountDividend = new AppUserCountDividend();
                                    appUserCountDividend.setUserId(userAgent.getUserId());
                                    appUserCountDividend.setDateNo(DateUtil.format(endTime, "yyyy-MM-dd"));
                                    appUserCountDividend.setBeginTime(beginTime);
                                    appUserCountDividend.setEndTime(endTime);
                                    appUserCountDividend.setProfitLoss(money);
                                    appUserCountDividend.setRate(rate);
                                    appUserCountDividend.setCoin(0.00);
                                    appUserCountDividend.setDiamondCoin(dividends);
                                    appUserCountDividend.setStatus(0);
                                    appUserCountDividendService.insertAppUserCountDividend(appUserCountDividend);
                                });
                            } else {
                                log.info("用户{}，所有下级亏损{}，查询下级为空，停止分红", appUser.getUserid(), money);
                            }
                        } else {
                            log.info("用户{}，所有下级亏损{}，不满足分红条件", appUser.getUserid(), money);
                        }
                    }
                });
            }
            log.info("每周代理分红完成");
        }catch (Exception e){
            log.info("每周分红失败：{}",e.getMessage());
        }

    }

    /**
     * 金额为负数则转为正数
     * */
    private double dealMoney(double money){
        return money<0?money*-1:money;
    }

}
